package com.aliyun.mts;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.mns.client.CloudAccount;
import com.aliyun.mns.client.CloudQueue;
import com.aliyun.mns.client.MNSClient;
import com.aliyun.mns.model.Message;
import com.aliyun.mts.domain.*;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.mts.model.v20140618.QueryJobListRequest;
import com.aliyuncs.mts.model.v20140618.QueryJobListResponse;
import com.aliyuncs.mts.model.v20140618.QueryJobListResponse.Job;
import com.aliyuncs.mts.model.v20140618.SubmitJobsRequest;
import com.aliyuncs.mts.model.v20140618.SubmitJobsResponse;
import com.aliyuncs.profile.DefaultProfile;
import org.apache.commons.lang.exception.ExceptionUtils;

import java.io.*;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class Main {
    //步骤1.请填写你所需访问的Region：cn-hangzhou、cn-shenzhen、cn-shanghai、cn-beijing
    private static final String MTS_REGION = "cn-hangzhou";
    private static final String OSS_REGION = "oss-" + MTS_REGION;

    //步骤2.设置mts的访问域名及oss的访问域名，注意所在区域不要设置错了
    private static final String mtsEndpoint = "mts." + MTS_REGION + ".aliyuncs.com";
    private static final String ossEndPoint = "http://oss-" + MTS_REGION + ".aliyuncs.com";


    private static String testVideoFilePath = "videos/test-中文.mp4";
    private static String waterMarkFilePath = "watermarks/32x32-中文.png";
    //步骤3.填写访问阿里云账号的AK，确保此账号已经开通MTS服务
    private static String accessKey = "";
    private static String accessKeySecret = "";


    //步骤4.设置管道ID，管道是作业排队用的，在MTS控制台全局设置管道列表中可以看到
    private static String pipelineId = "96c539f704d749e89304e19eadb86fb7";

    //步骤5.从阿里云消息服务的控制台对应区域Copy MNSEndPoint
    //MNS控制台链接：https://mns.console.aliyun.com
    private static final String mnsEndPoint = "http://1328410411576966.mns.cn-hangzhou.aliyuncs.com/";
    //步骤6.填写转码完成时消息通知队列，务必与管道上设置的消息队列名称保持的一致
    private static String transcodeCompletedNotifyQueueName = "testWater";


    //步骤7.设置转码模板ID，转码模板就是转码配置参数的集合，在MTS控制台全局设置转码模板列表中创建
    private static String transcodeTemplateId = "a68240fda66509f92e766fe54969bcbf";
    //步骤8.设置水印模板ID，在MTS控制台全局设置水印模板列表中创建
    private static String waterMarkTemplateId = "7dde7907260f48c8bd83160800fd484f";

    //步骤9.这是转码的输入bucket，存储视频源文件
    private static String inputBucket = "mts-console-test";

    //步骤10.设置转码输出bucket，存储转码完成后的视频文件
    private static String outputBucket = "output-zzz";

    private static CloudAccount account;
    private static CloudQueue queue;

    private static DefaultAcsClient aliyunClient;


    static {
        account = new CloudAccount(accessKey, accessKeySecret, mnsEndPoint);
        MNSClient mnsClient = account.getMNSClient();
        queue = mnsClient.getQueueRef(transcodeCompletedNotifyQueueName);
        try {
            DefaultProfile.addEndpoint(MTS_REGION, MTS_REGION, "Mts", mtsEndpoint);
        } catch (ClientException e) {
            System.out.print(ExceptionUtils.getStackTrace(e));
            System.exit(1);
        }
        aliyunClient = new DefaultAcsClient(DefaultProfile.getProfile(MTS_REGION, accessKey, accessKeySecret));
    }

    public static void main(String[] args) throws ClientException {

        try {
            //步骤11.上传视频源文件，若视频已经存在OSS上，则只需要获取到OSS地址也可以
            OSSFileDO inputFile = uploadVideoFile();
            //步骤12.若需要打水印，先将水印上传到OSS上，否则可以注释掉水印上传的代码
            OSSFileDO watermarkFile = uploadWatermarkFile();

            //步骤13.提交转码作业
            submitTranscodeJob(inputFile, watermarkFile, transcodeTemplateId);


            //步骤14.等待转码完成消息回调汇报转码完成

            while (true) {
                List<Message> messageList = queue.batchPopMessage(16, 30);
                if (null == messageList || messageList.isEmpty()) {
                    try {
                        Thread.sleep(20);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    continue;
                }
                for (Message message : messageList) {
                    handlingMessage(message);
                }


            }
        } catch (Exception e) {
            System.out.println(ExceptionUtils.getFullStackTrace(e));
            System.exit(1);
        }


    }

    private static void handlingMessage(Message message) {
        String messageBody = message.getMessageBody();
        JobDTO jobDTO = JSONObject.parseObject(messageBody, JobDTO.class);

        if (jobDTO.getJobId() == null || jobDTO.getState() == null || !jobDTO.getType().equals(JobType.Transcode.name())) {
            queue.deleteMessage(message.getReceiptHandle());
            return;
        }


        if (jobDTO.getState().equals(JobState.Success.name())) {
            Job transcodeJob = getTranscodeJobDetail(jobDTO.getJobId());

            String outputBucket = transcodeJob.getOutput().getOutputFile().getBucket();
            String outputObject = transcodeJob.getOutput().getOutputFile().getObject();
            String outputFileOSSUrl = getOSSUrl(outputBucket, outputObject);
            System.out.println("Transcode success, transcode output file url is " + outputFileOSSUrl);
        } else if (jobDTO.getState().equals(JobState.Fail.name())) {
            System.out.println("Transcode Fail,jobId:" + jobDTO.getJobId() + " Code:" + jobDTO.getCode() + " Msg:" + jobDTO.getMsg());
        }

        //如果是report消息，需要从队列中删除，否则会一直在队列中
        queue.deleteMessage(message.getReceiptHandle());


    }

    private static OSSFileDO uploadVideoFile() {
        return uploadLocalFile(inputBucket, "demo/video/" + UUID.randomUUID().toString() + "-中文.mp4", testVideoFilePath);
    }

    private static OSSFileDO uploadWatermarkFile() {
        return uploadLocalFile(inputBucket, "demo/watermarks/" + UUID.randomUUID().toString() + "-中文.png", waterMarkFilePath);
    }

    public static OSSFileDO uploadLocalFile(String bucket, String object, String filePath) {
        OSSClient client = new OSSClient(ossEndPoint, accessKey, accessKeySecret);
        try {
            String utf8FilePath = URLDecoder.decode(ClassLoader.getSystemClassLoader().getResource(filePath).getPath(),
                    "utf-8");
            File file = new File(utf8FilePath);
            InputStream content = new FileInputStream(file);

            ObjectMetadata meta = new ObjectMetadata();
            meta.setContentLength(file.length());
            client.putObject(bucket, object, content, meta);

            String encodedObjectName = URLEncoder.encode(object, "utf-8");
            return new OSSFileDO(OSS_REGION, bucket, encodedObjectName);
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }


    private static String getOSSUrl(String bucket, String object) {
        return "http://" + bucket + "." + OSS_REGION + ".aliyuncs.com/" + object;
    }


    private static String submitTranscodeJob(OSSFileDO inputFile, OSSFileDO watermarkFile, String templateId) {


        String outputOSSObjectKey = null;
        try {
            outputOSSObjectKey = URLEncoder.encode("demo/transcode/" + UUID.randomUUID().toString() + "-中文.mp4",
                    "utf-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }


        SubmitJobsRequest request = new SubmitJobsRequest();
        request.setInput(inputFile.toJsonString());
        request.setOutputBucket(outputBucket);
        List<Output> outputs = new ArrayList<Output>();

        List<WaterMark> watermarkList = new ArrayList<WaterMark>();
        watermarkList.add(new WaterMark(waterMarkTemplateId, watermarkFile, "100", "100", "10", "10", ReferPos.TopLeft.name()));
        outputs.add(new Output(templateId, outputOSSObjectKey, watermarkList));
        request.setOutputs(JSONArray.toJSONString(outputs));
        request.setPipelineId(pipelineId);
        request.setOutputLocation(OSS_REGION);

        Integer outputJobCount = 1;

        SubmitJobsResponse response = null;
        try {
            response = aliyunClient.getAcsResponse(request);
            if (response.getJobResultList().size() != outputJobCount) {
                throw new RuntimeException("SubmitJobsRequest Size failed");
            }

            return response.getJobResultList().get(0).getJob().getJobId();
        } catch (ServerException e) {
            throw new RuntimeException(e);
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    private static Job getTranscodeJobDetail(String transcodeJobId) {
        QueryJobListRequest request = new QueryJobListRequest();
        request.setJobIds(transcodeJobId);

        QueryJobListResponse response = null;
        try {
            response = aliyunClient.getAcsResponse(request);
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }

        Job transcodeJob = response.getJobList().get(0);
        return transcodeJob;


    }


}
